/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports

// Local imports
// None

/**
 * Interface representing the output of an audio pipeline..
 * <p>
 * Conceptually an  AudioDevice might run all speakers or just part of
 * the total soundscape.
 *
 * @author Alan Hudson
 * @version $Revision: 1.3 $
 */
public interface AudioDevice
{
    /**
     * Tell the surface to use the given thread as the rendering control thread.
     * Used when you want to clock the renderer to the rest of the pipeline,
     * rather than letting it sit in it's own loop. Due to
     * bugs in JOGL, we have to start the internal rendering threads after the
     * containing window has become visible. See JOGL issue #54 for more details
     * on this bug.
     *
     * @param th The render control thread instance to use or null
     */
    public void setRenderControlThread(Thread th);

    /**
     * An object has been detected as being deleted (no internal or
     * external references to it left) and it needs to be cleaned up. The
     * next oppourtunity this drawable and any sub-drawables get, this object
     * should have it's <code>cleanup()</code> method called and then the
     * reference to it should be discarded.
     *
     * @param obj The instance of the object to queue for deletion
     */
    public void queueDeletedObject(DeletableSceneGraphObject obj);

    /**
     * Update the list of items to be rendered to the current list. Draw them
     * at the next oppourtunity.
     *
     * @param commands The list of drawable surfaces to render
     */
    public void setDrawableObjects(AudioInstructions commands);

    /**
     * Instruct the surface to draw the collected set of nodes now. The
     * registered view environment is used to draw to this surface. If no
     * view is registered, the surface is cleared and then this call is
     * exited. The drawing surface does not swap the buffers at this point.
     */
    public void draw();

    /**
     * Get the underlying object that this surface is rendered to. If it is a
     * screen display device, the surface can be one of AWT Component or
     * Swing JComponent. An off-screen buffer would be a form of AWT Image etc.
     *
     * @return The drawable surface representation
     */
    public Object getSurfaceObject();

    /**
     * Instruct this surface that you have finished with the resources needed
     * and to dispose all rendering resources.
     */
    public void dispose();
}
