/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
// None

/**
 * A marker interface that represents a single complete audio rendering
 * pipeline.
 * <p>
 *
 * A pipeline represents all of the steps that may be accomplished
 * within a rendering cycle - culling, sorting and drawing. While an end-user
 * may wish to directly call the methods on this interface directly to control
 * their own rendering, it is recommended that a dedicated pipeline manager be
 * used for this task.
 * <p>
 *
 * If the pipeline does not have a audio device registered, it will still
 * complete all the steps up to that point. If no scene is registered, no
 * functionality is performed - render() will return immediately.
 *
 * @author Alan Hudson
 * @version $Revision: 1.4 $
 */
public interface AudioPipeline
{
    /**
     * Register an audio output device that this pipeline will send its output
     * to. Setting a null value will remove the current device.
     *
     * @param device The device instance to use or replace
     */
    public void setAudioDevice(AudioDevice device);

    /**
     * Get the currently registered audio device instance. If none is set,
     * return null.
     *
     * @return The currently set device instance or null
     */
    public AudioDevice getAudioDevice();

    /**
     * Start the pipeline functioning now. All steps will be called and this
     * method will not return until all are completed.
     */
    public void render();

    /**
     * Cause the device to re-emit the next frame only, with no processing of
     * the pipeline. This is typically an optimisation step when nothing has
     * changed in user land, so there's no processing that needs to be done.
     * Skip the processing and tell the drawable surface to render again what
     * it already has set from the previous frame.
     */
    public void displayOnly();

    /**
     * Set the root of the scene graph to be used by this pipeline. A value of
     * null will remove the scene from being rendered, causing the pipeline to
     * function as a no-op when rendered.
     *
     * @param scene The scene instance to render
     */
    public void setRenderableScene(Scene scene);

    /**
     * Force a halt of the current processing. Any processing in progress
     * should exit immediately. Used to abort the current scene processing due
     * to application shutdown or complete scene replacement.
     */
    public void halt();
}
