/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
import java.util.HashMap;

// Local imports
// None

/**
 * Base collection of functionality marking background nodes of various types.
 * <p>
 *
 * Backgrounds are rendered as the first item but do not interact with the
 * normal geometry in the rendering process. Typically, backgrounds are
 * rendered in a fixed volume (a unit box or sphere is the most common) with
 * depthbuffer reads and writes disabled. Ordinary geometry is then drawn over
 * the top.  Backgrounds must fit within clipping planes of [0.1,1].
 * <p>
 * As such, backgrounds are not typically subject to most rendering effects,
 * such as lighting, fog, perspective projection etc.
 *
 * @author Justin Couch
 * @version $Revision: 1.8 $
 */
public abstract class Background extends Leaf
    implements RenderableObject
{
    /** Map of display contexts to maps */
    protected HashMap dispListMap;

    /**
     * Constructs a background node
     */
    protected Background()
    {
    }

    //----------------------------------------------------------
    // Methods defined by Node
    //----------------------------------------------------------

    /**
     * Mark this node as having dirty bounds due to one of it's children having
     * their bounds changed.
     */
    protected void markBoundsDirty()
    {
        // Bounds are not used by backgrounds
    }
}
