/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
import java.util.ArrayList;

import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;
import java.nio.ByteBuffer;
import net.java.games.joal.AL;

// Local imports
// None

/**
 * A BackgroundSound class whichs emits a sound which doesn't change
 * by distance or orientation.
 * <p>
 *
 * @author Alan Hudson
 * @version $Revision: 1.4 $
 */
public class BackgroundSound extends Sound
{
    /** The OpenAL source */
    int source;

    /** BackgroundSound Position */
    private Point3f position;

    /** Scratch point */
    private Point3f tmpPoint;

    /**
     * Creates a sound.
     */
    public BackgroundSound()
    {
        position = new Point3f();
        tmpPoint = new Point3f();
    }

    /**
     * This method is called to render this node.  All openGL commands needed
     * to render the node should be executed.  Any transformations needed
     * should be added to the transformation stack premultiplied.  This
     * method must be re-entrant as it can be called from multiple
     * places at once.
     *
     * @param al The al context to render with
     * @param transform The transformation stack to this node
     */
    public void render(AL al, Matrix4f transform)
    {
        if (dataChanged)
        {
            buffer = soundSource.getBufferId(al, seq);

            if (buffer == -1)
                return;

            dataChanged = false;
            playChanged = true;

            int[] sources = new int[1];

            // Bind buffer with a source.
            al.alGenSources(1, sources);
            source = sources[0];

            if (al.alGetError() != AL.AL_NO_ERROR)
                System.out.println("Error creating OpenAL Source");

            al.alSourcei(source, AL.AL_BUFFER, buffer);
            al.alSourcef(source, AL.AL_GAIN, 1.0f);

            al.alSource3f(source, AL.AL_POSITION, tmpPoint.x, tmpPoint.y,tmpPoint.z);
            al.alSourcei(source, AL.AL_SOURCE_RELATIVE, AL.AL_TRUE);
            al.alSourcei(source, AL.AL_LOOPING, loop ? 1 : 0);
            al.alSourcef(source, AL.AL_PITCH, pitch);

            if (al.alGetError() != AL.AL_NO_ERROR)
                System.out.println("Error setting OpenAL Source params");
        }

        if (paramsChanged)
        {
            al.alSourcef(source, AL.AL_PITCH, pitch);
            al.alSourcei(source, AL.AL_LOOPING, loop ? 1 : 0);

            paramsChanged = false;
        }

        if (playChanged)
        {
            if (playing && paused)
            {
                al.alSourcePlay(source);
            }

            if (playing)
            {
                if (paused)
                    al.alSourcePause(source);
                else {
                    al.alSourcePlay(source);
                }
            } else
                al.alSourceStop(source);

            playChanged = false;
        } else if (playing)
        {
            int state = al.alGetSourcei(source, AL.AL_SOURCE_STATE);
            if (state == AL.AL_STOPPED)
            {
                playing = false;
            }
        }
    }

    /**
     * This method is called after a node has been rendered.  This method
     * must be re-entrant.
     *
     * @param al The al context to draw with
     */
    public void postRender(AL al)
    {
    }

    /**
     * State check to see whether the sound is enabled.
     * @return true if the sound has something to render
     */
    public boolean isEnabled()
    {
        return true;
    }
}
