/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
import java.util.HashMap;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

// Local imports
// None

/**
 * Base class representing a single shader program that can be applied to the
 * rendering pipeline.
 * <p>
 *
 * @author Justin Couch
 * @version $Revision: 1.4 $
 */
public abstract class GL14ShaderProgram extends NodeComponent
{
    /** The program string that is stored as the shader */
    protected String programString;

    /** Flag to say that the string has changed and should be recompiled */
    protected boolean programChanged;

    /** The ID of the shader program that has been allocated here */
    protected int programId;

    /**
     * Constructs a Shader with default values.
     */
    public GL14ShaderProgram()
    {
    }

    /**
     * Set the program string that is to be registered by this shader. Setting
     * a value of null will clear the current shader and prevent it from being
     * rendered next frame.
     *
     * @param str The new program string to be registered or null
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the NodeUpdateListener callback method
     */
    public void setProgramString(String str)
        throws InvalidWriteTimingException
    {
        if(isLive() && updateHandler != null &&
           !updateHandler.isDataWritePermitted(this))
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        if(programString != str)
        {
            programChanged = true;
            programString = str;
        }
    }

    /**
     * Get the currently set program string. If none is set, return null.
     *
     * @return The current string or null
     */
    public String getProgramString()
    {
        return programString;
    }
}
