/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2006
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
import java.util.HashMap;

import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

// Local imports
// None

/**
 * An OpenGL LineArray.
 * <p>
 *
 * Lines cannot be picked in the traditional manner using geometry-based
 * picking. The only way to pick them is based on their bounds.
 *
 * <h3>Setting geometry</h3>
 *
 * <p>Part of the optimisation we make is to only copy into the underlying
 * structures the exact number of coordinates, normals etc that are needed.
 * To know this number, we need to know how many coordinates exist before
 * attempting to set anything else. When constructing, or updating, geometry,
 * you should always make sure that you first set the vertex list, then the
 * sizing information for the strip or fan counts, and then set normals as
 * needed. </p>
 *
 * @author Justin Couch
 * @version $Revision: 1.10.2.1 $
 */
public class IndexedLineArray extends IndexedVertexGeometry
{
    /**
     * Constructs a LineArray with default values.
     */
    public IndexedLineArray()
    {
    }

    //----------------------------------------------------------
    // Methods defined by RenderableObject
    //----------------------------------------------------------

    /**
     * Issue ogl commands needed for this renderable object.
     *
     * @param gl The gl context to draw with
     * @param glu The utility context to draw with
     */
    public void render(GL gl, GLU glu)
    {
        // No coordinates, do nothing.
        if((vertexFormat & COORDINATE_MASK) == 0)
            return;

        setVertexState(gl, glu);

        gl.glDrawElements(GL.GL_LINES,
                          numIndices,
                          GL.GL_UNSIGNED_INT,
                          indices);

        clearVertexState(gl, glu);
    }

    /**
     * Restore all openGL state.
     *
     * @param gl The gl context to draw with
     * @param glu The utility context to draw with
     */
    public void postRender(GL gl, GLU glu)
    {
    }
}
