/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
import net.java.games.jogl.GL;
import net.java.games.jogl.GLU;

/**
 * A single layer of a composited set of scenes to form a complete visual
 * system.
 * <p>
 *
 * <b>Note: This class does nothing in Aviatrix3D 1.0</b>
 * <p>
 *
 * A layer is a composite of objects that are applied in a sequential manner
 * to the given surface. Between each layer the depth buffer is cleared and
 * a new rendering is applied directly over the top of the previous. Colour
 * buffers or other buffers are not cleared.
 *
 * @author Justin Couch
 * @version $Revision: 1.4 $
 */
public class Layer extends SceneGraphObject
{
    /** The scene that this layer should render */
    private Scene scene;

    /**
     * Construct a new layer instance that does not have a pre-assigned scene.
     */
    public Layer()
    {
    }

    //----------------------------------------------------------
    // Methods defined by ScenegraphObject
    //----------------------------------------------------------

    /**
     * Set the scenegraph update handler for this node.  It will notify
     * all its children of the value. A null value will clear the current
     * handler.
     *
     * @param handler The instance to use as a handler
     */
    protected void setUpdateHandler(NodeUpdateHandler handler)
    {
        super.setUpdateHandler(handler);

        if(scene != null)
            scene.setUpdateHandler(handler);
    }

    /**
     * Notification that this object is live now. Overridden to make sure that
     * the live state of the nodes represents the same state as the parent
     * scene graph.
     *
     * @param state true if this should be marked as live now
     */
    protected void setLive(boolean state)
    {
        super.setLive(state);

        if(scene != null)
            scene.setLive(state);
    }

    //----------------------------------------------------------
    // Local Methods
    //----------------------------------------------------------

    /**
     * Set the scene to be used by this layer. If a previous scene was used, it
     * is replaced by this scene. Note that a scene can not have more than one
     * parent, so sharing it between layers is not an option.
     *
     * @param sc The scene instance to use
     * @throws InvalidWriteTimingException An attempt was made to write outside
     *   of the NodeUpdateListener callback method
     */
    public void setScene(Scene sc)
        throws InvalidWriteTimingException
    {
        if(isLive() && updateHandler != null &&
           !updateHandler.isDataWritePermitted(this))
            throw new InvalidWriteTimingException(WRITE_TIMING_MSG);

        if(scene != null)
            scene.setLive(false);

        scene = sc;

        if(scene != null)
        {
            scene.setUpdateHandler(updateHandler);
            scene.setLive(alive);
        }
    }

    /**
     * Get the currently set scene instance. If no scene is set, null is
     * returned.
     *
     * @return The current scene instance or null
     */
    public Scene getScene()
    {
        return scene;
    }
}
