/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
import java.awt.Rectangle;

import javax.vecmath.Matrix4f;

// Local imports
// None

/**
 * Data holder class used to pass the current environment data along the rendering
 * pipeline.
 * <p>
 *
 * End-user code should never be making use of this class unless the end user is
 * implementing a customised rendering pipeline. This class is used as a simple
 * internal collection of the per-frame renderable data that is passed along
 * each stage of the pipeline.
 *
 * @author Justin Couch
 * @version $Revision: 1.9 $
 */
public class RenderEnvironmentData
{
    /** Any user provided data registered with the effects processor */
    public Object userData;

    /** If set, use this to do pre and post rendered effects */
    public RenderEffectsProcessor effectsProcessor;

    /** The current viewpoint instance */
    public Viewpoint viewpoint;

    /** The current fog instance */
    public Fog fog;

    /**
     * Matrix representing the transformation of the background node from
     * the root of the scene to it's location. It already has the translation
     * components removed by the time it is set here.
     */
    public Matrix4f backgroundTransform;

    /** The current background instance */
    public Background background;

    /** Current view environment */
    public ViewEnvironment viewEnvironment;

    /** ViewFrustum positions.  left,right,bottom,top,near,far */
    public double[] viewFrustum;

    /** Matrix representing the view frustum transformation */
    public Matrix4f viewTransform;

    /** Matrix representing the projection transform */
    public Matrix4f projectionMatrix;

    /**
     * The amount of eye offset in use from the nominal middle. This is a
     * convenience data item to save the need to regenerate it at the renderer
     * level.
     */
    public float[] eyeOffset;

    /** Flag indicating if the current viewport is valid */
    public boolean hasViewport;

    /** The current dimensions of the viewport, if set */
    public Rectangle viewport;

    /**
     * Create a new instance of this class. The variables are initialized
     * to their default values and arrays constructed.
     */
    public RenderEnvironmentData()
    {
        hasViewport = false;
        viewport = new Rectangle();
        eyeOffset = new float[3];
        viewFrustum = new double[6];
        viewTransform = new Matrix4f();
        projectionMatrix = new Matrix4f();
        backgroundTransform = new Matrix4f();
    }
}
