/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
// None

/**
 * Class for passing the detailed rendering information for a single surface.
 * from the sorter through to the renderable surface.
 * <p>
 *
 * Since pBuffers cannot be shared across different GL contexts with the
 * construct that JOGL is is using, when two parent textures reference a
 * single child, there will be two copies of this class, with two
 * different parentSource references, but only one copyOf set.
 *
 * @author Justin Couch
 * @version $Revision: 1.7 $
 */
public class RenderInstructions
{
    /** The initial size of the children list */
    private static final int LIST_START_SIZE = 496;

    /** Visual data such as viewpoint, background etc */
    public RenderEnvironmentData renderData;

    /**
     * If set, the rendering should be done to an offscreen buffer that is
     * then handed to this texture placeholder for use in another drawable
     * object. If this is null, then render to the main on-screen drawable
     * object.
     */
    public PBufferTextureSource pbuffer;

    /**
     * Reference to the real instance of this class that contains the
     * renderList and renderOps you should read from. If null, this is the
     * original specification.
     */
    public RenderInstructions copyOf;

    /**
     * Reference to the parent texture source that this instance is to be
     * rendered into. If the parent source is the root canvas (ie main
     * drawable), this reference will be null.
     */
    public OffscreenTextureSource parentSource;

    /** The list of nodes in sorted order */
    public RenderDetails[] renderList;

    /** Operation to perform on each node */
    public int[] renderOps;

    /** The number of valid items in the array */
    public int numValid;

    /**
     * Construct a new instance of this class with the arrays initialised
     * to a default size.
     */
    public RenderInstructions()
    {
        renderList = new RenderDetails[LIST_START_SIZE];
        renderOps = new int[LIST_START_SIZE];

        for(int i = 0; i < LIST_START_SIZE; i++)
            renderList[i] = new RenderDetails();
    }

}
