/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
// None

/**
 * Constants used to define render operations as the output of the sort stage.
 * <p>
 *
 * @author Justin Couch
 * @version $Revision: 1.12 $
 */
public interface RenderOp
{
    /** The unknown/general request to push state onto the stack */
    public static final int START_RENDER = 1;

    /** The unknown/general request to pop state off the stack */
    public static final int STOP_RENDER = 2;

    /**
     * Render a geometry item. Since this is a terminal for the OpenGL pipeline
     * state, no postRender call will be made.
     */
    public static final int RENDER_GEOMETRY = 3;

    /**
     * A node implementing the {@link CustomRenderable} interface will be
     * rendered. Needs to pass in the external instruction setup.
     */
    public static final int RENDER_CUSTOM = 4;

    /** The node component should push its state onto the stack */
    public static final int START_STATE = 5;

    /** The node component should remove its state from the stack */
    public static final int STOP_STATE = 6;

    /** A light should push its state onto the stack */
    public static final int START_LIGHT = 9;

    /** A light is removing its state from the stack */
    public static final int STOP_LIGHT = 10;

    /** Start of a collection of transparent geometry */
    public static final int START_TRANSPARENT = 11;

    /** End of a collection of transparent geometry */
    public static final int STOP_TRANSPARENT = 12;

    /** Start of a collection of objects that are used for shadows */
    public static final int START_SHADOW = 13;

    /** End of a collection of objects that are used for shadows */
    public static final int STOP_SHADOW = 14;

    /** Start of a collection of objects that are shadow generators */
    public static final int START_SHADOW_GENERATOR = 15;

    /** End of a collection of objects that are shadow generators */
    public static final int STOP_SHADOW_GENERATOR = 16;

    /** A clip plane should push its state onto the stack */
    public static final int START_CLIP_PLANE = 17;

    /** A clip plane is removing its state from the stack */
    public static final int STOP_CLIP_PLANE = 18;

    /** A local fog should push its state onto the stack */
    public static final int START_FOG = 19;

    /** A local fog is removing its state from the stack, restore global fog. */
    public static final int STOP_FOG = 20;

    /** Start a GLSLang shader program now */
    public static final int START_SHADER_PROGRAM = 21;

    /** Stop using a GLSLang shader programs */
    public static final int STOP_SHADER_PROGRAM = 22;

    /** Set a collection of shader arguments */
    public static final int SET_SHADER_ARGS = 23;

    /**
     * Turn on a texture unit stage for rendering. The render instruction
     * will be accompanied by an Integer instance that holds the stage ID
     * to be used for this unit as it needs to be rendered right now.
     */
    public static final int START_TEXTURE = 24;

    /**
     * Turn off a texture unit stage for rendering. See START_TEXTURE for
     * the details.
     */
    public static final int STOP_TEXTURE = 25;
}
