/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d;

// External imports
// None

// Local imports
// None

/**
 * Representation of all the culled structure information that should be
 * applied to a single render target (surface).
 * <p>
 *
 * A renderable target is either the raw surface or a pBuffer-based texture.
 * These may be combined together to generate the single output scene. This
 * class represents all of the structures needed to handle a single scene,
 * not including any other referenced targets (it is assumed that the culling
 * stage will order these correctly so that one target will have all of it's
 * supplied dependencies already created.
 * <p>
 *
 * From a downstream rendering perspective, the rendering order is to process
 * each multipass rendering section, clearing all buffers each time, followed
 * by a final clearing of all buffers and finally the nodes of this target,
 * starting with the deepest layer first.
 *
 * @author Justin Couch
 * @version $Revision: 1.2 $
 */
public class RenderableTarget
{
    /**
     * The list of multipass renders that are needed to be performed on this
     * target.
     */
    public MultipassRenderableTarget[] multipassTargets;

    /** The number of valid items to use from the targets */
    public int numMultipassTargets;

    /** The list of culled nodes to use on this render target */
    public CulledOutputDetails[] nodes;

    /** The number of valid values in the nodes list */
    public int numNodes;

    /** The local data about the rendered environment */
    public RenderEnvironmentData envData;

    /** List of objects that are considered the parent of this target */
    public SceneGraphObject[] sceneParents;

    /** THe number of valid sceneParents to use */
    public int numParents;
}
