/*****************************************************************************
 *                     Yumetech, Inc Copyright (c) 2004-2005
 *                               Java Source
 *
 * This source is licensed under the GNU LGPL v2.1
 * Please read http://www.gnu.org/copyleft/lgpl.html for more information
 *
 * This software comes with the standard NO WARRANTY disclaimer for any
 * purpose. Use it at your own risk. If there's a problem you get to fix it.
 *
 ****************************************************************************/

package org.j3d.aviatrix3d.pipeline;

// External imports
// None

// Local imports
import org.j3d.aviatrix3d.CulledGeometryReceiver;
import org.j3d.aviatrix3d.CullOutputDetails;
import org.j3d.aviatrix3d.RenderEnvironmentData;
import org.j3d.aviatrix3d.SceneGraphObject;
import org.j3d.aviatrix3d.SortStage;

/**
 * Adapter class that maps the output of a cull stage to a single sorter.
 * <p>
 *
 * @author Justin Couch
 * @version $Revision: 1.8 $
 */
public class CullToSingleSortListener implements CulledGeometryReceiver
{
    /** The sorter to hand the nodes to */
    private SortStage sorter;

    /**
     * Create a new default instance of this class with nothing set.
     */
    public CullToSingleSortListener()
    {
    }

    //---------------------------------------------------------------
    // Methods defined by CulledGeometryReceiver
    //---------------------------------------------------------------

    /**
     * Sort the listing of nodes in the given array. Do not return until the
     * sort has been completed. If the sceneParent is the main scene graph and
     * not an offscreen texture, then that spot should be set to null in the list.
     * Here's the sorted output list of nodes. For the 2D array of objects, contains
     * the list of final subscenes to send to the final stage. First index is
     * the direct owner of the scene contents. The second index is the
     * scene parent of the scene included (needed for pBuffer GL context
     * handling at render time). If this second one is null, then the parent
     * is the main canvas that is being rendered to.
     *
     * @param data External rendering environment information
     * @param nodes List of processed nodes based on the scene they came from
     * @param numNodes Number of nodes in each scene
     * @param sceneParents Parent nodes that hold the subscene in the main scene graph
     * @param numScenes The number of valid scenes to process
     */
    public void culledOutput(RenderEnvironmentData[] data,
                             CullOutputDetails[][] nodes,
                             int[] numNodes,
                             SceneGraphObject[][] sceneParents,
                             int numScenes)
    {
        // Need to fix the buffer ID setup here.
        if(sorter != null)
            sorter.sort(data, nodes, numNodes, sceneParents, numScenes, 0);
    }

    //---------------------------------------------------------------
    // Local methods
    //---------------------------------------------------------------

    /**
     * Set the sorter instance to be used as the target. Passing a value of
     * null will clear the current registered instance.
     *
     * @param s The sorter instance to use or null
     */
    public void setSorter(SortStage s)
    {
        sorter = s;
    }
}
